@@ -1,12 +1,12 @@
 #
 #               tk/spinbox.rb - Tk spinbox classes
-#                       $Date: 2005/08/09 06:16:00 $
+#                       $Date: 2007-12-21 17:57:35 +0900 (Fri, 21 Dec 2007) $
 #                       by Yukihiro Matsumoto <matz@caelum.co.jp>
 #
 require 'tk'
 require 'tk/entry'
 
-class TkSpinbox<TkEntry
+class Tk::Spinbox<TkEntry
   TkCommandNames = ['spinbox'.freeze].freeze
   WidgetClassName = 'Spinbox'.freeze
   WidgetClassNames[WidgetClassName] = self
@@ -64,6 +64,21 @@ class TkSpinbox<TkEntry
   #end
   #private :create_self
 
+  def __boolval_optkeys
+    super() << 'wrap'
+  end
+  private :__boolval_optkeys
+
+  def __strval_optkeys
+    super() << 'buttonbackground' << 'format'
+  end
+  private :__strval_optkeys
+
+  def __listval_optkeys
+    super() << 'values'
+  end
+  private :__listval_optkeys
+
   def identify(x, y)
     tk_send_without_enc('identify', x, y)
   end
@@ -82,3 +97,5 @@ class TkSpinbox<TkEntry
     _fromUTF8(tk_send_without_enc('set', _get_eval_enc_str(str)))
   end
 end
+
+TkSpinbox = Tk::Spinbox unless Object.const_defined? :TkSpinbox
