@@ -22,7 +22,7 @@ class TkTextTag<TkObject
 
   def initialize(parent, *args)
     #unless parent.kind_of?(TkText)
-    #  fail ArguemntError, "expect TkText for 1st argument"
+    #  fail ArgumentError, "expect TkText for 1st argument"
     #end
     @parent = @t = parent
     @tpath = parent.path
@@ -51,7 +51,8 @@ class TkTextTag<TkObject
   end
 
   def exist?
-    if ( tk_split_simplelist(_fromUTF8(tk_call_without_enc(@t.path, 'tag', 'names'))).find{|id| id == @id } )
+    #if ( tk_split_simplelist(_fromUTF8(tk_call_without_enc(@t.path, 'tag', 'names'))).find{|id| id == @id } )
+    if ( tk_split_simplelist(tk_call_without_enc(@t.path, 'tag', 'names'), false, true).find{|id| id == @id } )
       true
     else
       false
@@ -173,7 +174,7 @@ class TkTextTag<TkObject
   #end
   def bind(seq, *args)
     # if args[0].kind_of?(Proc) || args[0].kind_of?(Method)
-    if TkComm._callback_entry?(args[0])
+    if TkComm._callback_entry?(args[0]) || !block_given?
       cmd = args.shift
     else
       cmd = Proc.new
@@ -188,7 +189,7 @@ class TkTextTag<TkObject
   #end
   def bind_append(seq, *args)
     # if args[0].kind_of?(Proc) || args[0].kind_of?(Method)
-    if TkComm._callback_entry?(args[0])
+    if TkComm._callback_entry?(args[0]) || !block_given?
       cmd = args.shift
     else
       cmd = Proc.new
@@ -224,6 +225,7 @@ class TkTextTag<TkObject
     self
   end
 end
+TktTag = TkTextTag
 
 class TkTextNamedTag<TkTextTag
   def self.new(parent, name, *args)
@@ -247,7 +249,7 @@ class TkTextNamedTag<TkTextTag
 
   def initialize(parent, name, *args)
     #unless parent.kind_of?(TkText)
-    #  fail ArguemntError, "expect TkText for 1st argument"
+    #  fail ArgumentError, "expect TkText for 1st argument"
     #end
     @parent = @t = parent
     @tpath = parent.path
@@ -270,9 +272,11 @@ class TkTextNamedTag<TkTextTag
     @t._addtag id, self
   end
 end
+TktNamedTag = TkTextNamedTag
 
 class TkTextTagSel<TkTextNamedTag
   def self.new(parent, *args)
     super(parent, 'sel', *args)
   end
 end
+TktTagSel = TkTextTagSel
