@@ -10,7 +10,7 @@
 #
 # U1)
 #   class Foo
-#     extend Exception2MassageMapper
+#     extend Exception2MessageMapper
 #     def_e2message ExistingExceptionClass, "message..."
 #     def_exception :NewExceptionClass, "message..."[, superclass]
 #     ...
@@ -18,13 +18,13 @@
 #
 # U2)
 #   module Error
-#     extend Exception2MassageMapper
+#     extend Exception2MessageMapper
 #     def_e2meggage ExistingExceptionClass, "message..."
 #     def_exception :NewExceptionClass, "message..."[, superclass]
 #     ...
 #   end
 #   class Foo
-#     include Exp
+#     include Error
 #     ...
 #   end
 #
@@ -33,7 +33,7 @@
 #
 # U3)
 #   module Error
-#     extend Exception2MassageMapper
+#     extend Exception2MessageMapper
 #     def_e2message ExistingExceptionClass, "message..."
 #     def_exception :NewExceptionClass, "message..."[, superclass]
 #     ...
@@ -48,8 +48,6 @@
 #   Foo.Fail ExistingExceptionClass, arg...
 #
 #
-fail "Use Ruby 1.1" if VERSION < "1.1"
-
 module Exception2MessageMapper
   @RCS_ID='-$Id: e2mmap.rb,v 1.10 1999/02/17 12:33:17 keiju Exp keiju $-'
 
@@ -57,24 +55,17 @@ module Exception2MessageMapper
 
   def E2MM.extend_object(cl)
     super
-    cl.bind(self) unless cl == E2MM
+    cl.bind(self) unless cl < E2MM
   end
   
-  # backward compatibility
-  def E2MM.extend_to(b)
-    c = eval("self", b)
-    c.extend(self)
-  end
-
   def bind(cl)
     self.module_eval %[
       def Raise(err = nil, *rest)
-	Exception2MessageMapper.Raise(self.type, err, *rest)
+	Exception2MessageMapper.Raise(self.class, err, *rest)
       end
       alias Fail Raise
 
-      def self.append_features(mod)
-	super
+      def self.included(mod)
 	mod.extend Exception2MessageMapper
       end
     ]
@@ -89,20 +80,6 @@ module Exception2MessageMapper
   end
   alias Fail Raise
 
-  # backward compatibility
-  alias fail! fail
-  def fail(err = nil, *rest)
-    begin 
-      E2MM.Fail(self, err, *rest)
-    rescue E2MM::ErrNotRegisteredException
-      super
-    end
-  end
-  class << self
-    public :fail
-  end
-
-  
   # def_e2message(c, m)
   #	    c:  exception
   #	    m:  message_form
@@ -112,7 +89,7 @@ module Exception2MessageMapper
     E2MM.def_e2message(self, c, m)
   end
   
-  # def_exception(c, m, s)
+  # def_exception(n, m, s)
   #	    n:  exception_name
   #	    m:  message_form
   #	    s:	superclass(default: StandardError)
@@ -139,7 +116,7 @@ module Exception2MessageMapper
     c
   end
   
-  # E2MM.def_exception(k, c, m, s)
+  # E2MM.def_exception(k, n, m, s)
   #	    k:  class to define exception under.
   #	    n:  exception_name
   #	    m:  message_form
@@ -160,12 +137,11 @@ module Exception2MessageMapper
   #
   def E2MM.Raise(klass = E2MM, err = nil, *rest)
     if form = e2mm_message(klass, err)
-      $! = err.new(sprintf(form, *rest))
-      $@ = caller(1) if $@.nil?
+      b = $@.nil? ? caller(1) : $@
       #p $@
       #p __FILE__
-      $@.shift if $@[0] =~ /^#{Regexp.quote(__FILE__)}:/
-      raise
+      b.shift if b[0] =~ /^#{Regexp.quote(__FILE__)}:/
+      raise err, sprintf(form, *rest), b
     else
       E2MM.Fail E2MM, ErrNotRegisteredException, err.inspect
     end
