@@ -10,30 +10,12 @@
 #   
 #
 
-require "rational.rb"
 require "complex.rb"
+require "rational.rb"
 require "matrix.rb"
 
 class Integer
 
-  def gcd2(int)
-    a = self.abs
-    b = int.abs
-    a, b = b, a if a < b
-    
-    pd_a = a.prime_division
-    pd_b = b.prime_division
-    
-    gcd = 1
-    for pair in pd_a
-      as = pd_b.assoc(pair[0])
-      if as
-	gcd *= as[0] ** [as[1], pair[1]].min
-      end
-    end
-    return gcd
-  end
-  
   def Integer.from_prime_division(pd)
     value = 1
     for prime, index in pd
@@ -43,6 +25,7 @@ class Integer
   end
   
   def prime_division
+    raise ZeroDivisionError if self == 0
     ps = Prime.new
     value = self
     pv = []
@@ -67,34 +50,54 @@ end
   
 class Prime
   include Enumerable
+  # These are included as class variables to cache them for later uses.  If memory
+  #   usage is a problem, they can be put in Prime#initialize as instance variables.
 
-  def initialize
-    @seed = 1
-    @primes = []
-    @counts = []
+  # There must be no primes between @@primes[-1] and @@next_to_check.
+  @@primes = [2, 3, 5, 7, 11, 13, 17, 19, 23, 29, 31, 37, 41, 43, 47, 53, 59, 61, 67, 71, 73, 79, 83, 89, 97, 101]
+  # @@next_to_check % 6 must be 1.  
+  @@next_to_check = 103            # @@primes[-1] - @@primes[-1] % 6 + 7
+  @@ulticheck_index = 3            # @@primes.index(@@primes.reverse.find {|n|
+                                   #   n < Math.sqrt(@@next_to_check) })
+  @@ulticheck_next_squared = 121   # @@primes[@@ulticheck_index + 1] ** 2
+
+  class << self
+    # Return the prime cache.
+    def cache
+      return @@primes
+    end
+    alias primes cache
+    alias primes_so_far cache
   end
   
+  def initialize
+    @index = -1
+  end
+  
+  # Return primes given by this instance so far.
+  def primes
+    return @@primes[0, @index + 1]
+  end
+  alias primes_so_far primes
+  
   def succ
-    i = -1
-    size = @primes.size
-    while i < size
-      if i == -1
-	@seed += 1
-	i += 1
-      else
-	while @seed > @counts[i]
-	  @counts[i] += @primes[i]
-	end
-	if @seed != @counts[i]
-	  i += 1
-	else
-	  i = -1
-	end
+    @index += 1
+    while @index >= @@primes.length
+      # Only check for prime factors up to the square root of the potential primes,
+      #   but without the performance hit of an actual square root calculation.
+      if @@next_to_check + 4 > @@ulticheck_next_squared
+        @@ulticheck_index += 1
+        @@ulticheck_next_squared = @@primes.at(@@ulticheck_index + 1) ** 2
       end
+      # Only check numbers congruent to one and five, modulo six. All others
+      #   are divisible by two or three.  This also allows us to skip checking against
+      #   two and three.
+      @@primes.push @@next_to_check if @@primes[2..@@ulticheck_index].find {|prime| @@next_to_check % prime == 0 }.nil?
+      @@next_to_check += 4
+      @@primes.push @@next_to_check if @@primes[2..@@ulticheck_index].find {|prime| @@next_to_check % prime == 0 }.nil?
+      @@next_to_check += 2 
     end
-    @primes.push @seed
-    @counts.push @seed + @seed
-    return @seed
+    return @@primes[@index]
   end
   alias next succ
 
@@ -106,33 +109,30 @@ class Prime
 end
 
 class Fixnum
-  alias divmod! divmod
-  alias / rdiv
-  def divmod(other)
-    a = self.div(other)
-    b = self % other
-    return a,b
-  end
+  remove_method :/
+  alias / quo
 end
 
 class Bignum
-  alias divmod! divmod
-  alias / rdiv
+  remove_method :/
+  alias / quo
 end
 
 class Rational
   Unify = true
 
+  remove_method :inspect
   def inspect
-    format "%s/%s", @numerator.inspect, @denominator.inspect
+    format "%s/%s", numerator.inspect, denominator.inspect
   end
 
   alias power! **
 
   def ** (other)
     if other.kind_of?(Rational)
+      other2 = other
       if self < 0
-	return Complex(self, 0) ** other
+	return Complex.new!(self, 0) ** other
       elsif other == 0
 	return Rational(1,1)
       elsif self == 0
@@ -141,8 +141,8 @@ class Rational
 	return Rational(1,1)
       end
       
-      npd = @numerator.prime_division
-      dpd = @denominator.prime_division
+      npd = numerator.prime_division
+      dpd = denominator.prime_division
       if other < 0
 	other = -other
 	npd, dpd = dpd, npd
@@ -151,7 +151,7 @@ class Rational
       for elm in npd
 	elm[1] = elm[1] * other
 	if !elm[1].kind_of?(Integer) and elm[1].denominator != 1
-	  return Float(self) ** other
+         return Float(self) ** other2
 	end
 	elm[1] = elm[1].to_i
       end
@@ -159,7 +159,7 @@ class Rational
       for elm in dpd
 	elm[1] = elm[1] * other
 	if !elm[1].kind_of?(Integer) and elm[1].denominator != 1
-	  return Float(self) ** other
+         return Float(self) ** other2
 	end
 	elm[1] = elm[1].to_i
       end
@@ -171,11 +171,11 @@ class Rational
       
     elsif other.kind_of?(Integer)
       if other > 0
-	num = @numerator ** other
-	den = @denominator ** other
+	num = numerator ** other
+	den = denominator ** other
       elsif other < 0
-	num = @denominator ** -other
-	den = @numerator ** -other
+	num = denominator ** -other
+	den = numerator ** -other
       elsif other == 0
 	num = 1
 	den = 1
@@ -215,11 +215,11 @@ class Rational
       
     elsif other.kind_of?(Integer)
       if other > 0
-	num = @numerator ** other
-	den = @denominator ** other
+	num = numerator ** other
+	den = denominator ** other
       elsif other < 0
-	num = @denominator ** -other
-	den = @numerator ** -other
+	num = denominator ** -other
+	den = numerator ** -other
       elsif other == 0
 	num = 1
 	den = 1
@@ -235,6 +235,7 @@ class Rational
 end
 
 module Math
+  remove_method(:sqrt)
   def sqrt(a)
     if a.kind_of?(Complex)
       abs = sqrt(a.real*a.real + a.image*a.image)
