@@ -13,7 +13,6 @@
 #   	by Keiju ISHITSUKA (Nippon Rational Inc.)
 #
 
-require "tracer"
 require "e2mmap"
 require "irb/slex"
 
@@ -24,7 +23,7 @@ require "rdoc/markup/simple_markup/preprocess"
 
 require "rdoc/parsers/parserfactory"
 
-$TOKEN_DEBUG = $DEBUG
+$TOKEN_DEBUG = $DEBUG_RDOC
 
 # Definitions of all tokens involved in the lexical analysis
 
@@ -39,7 +38,7 @@ module RubyToken
   
   class Token
     NO_TEXT = "??".freeze
-    attr :text
+    attr_accessor :text
 
     def initialize(line_no, char_no)
       @line_no = line_no
@@ -54,7 +53,7 @@ module RubyToken
       self
     end
 
-    attr_reader :line_no, :char_no, :text
+    attr_reader :line_no, :char_no
   end
 
   class TkNode < Token
@@ -116,7 +115,7 @@ module RubyToken
     when String, Symbol
       source = token.kind_of?(String) ? TkReading2Token : TkSymbol2Token
       if (tk = source[token]).nil?
-	IRB.fail TkReading2TokenNoKey, token
+        fail TkReading2TokenNoKey, token
       end
       tk = Token(tk[0], value) 
     else 
@@ -271,7 +270,7 @@ module RubyToken
   def RubyToken.def_token(token_n, super_token = Token, reading = nil, *opts)
     token_n = token_n.id2name unless token_n.kind_of?(String)
     if RubyToken.const_defined?(token_n)
-      IRB.fail AlreadyDefinedToken, token_n
+      fail AlreadyDefinedToken, token_n
     end
 
     token_c =  Class.new super_token
@@ -280,7 +279,7 @@ module RubyToken
  
     if reading
       if TkReading2Token[reading]
-	IRB.fail TkReading2TokenDuplicateError, token_n, reading
+        fail TkReading2TokenDuplicateError, token_n, reading
       end
       if opts.empty?
 	TkReading2Token[reading] = [token_c]
@@ -436,6 +435,7 @@ class RubyLex
   def_exception(:SyntaxError, "%s")
   
   include RubyToken
+  include IRB
 
   attr_reader :continue
   attr_reader :lex_state
@@ -466,11 +466,10 @@ class RubyLex
     @exception_on_syntax_error = true
   end
 
-  attr :skip_space, true
-  attr :read_auto_clean_up, true
-  attr :exception_on_syntax_error, true
-
-  attr :indent
+  attr_accessor :skip_space
+  attr_accessor :read_auto_clean_up
+  attr_accessor :exception_on_syntax_error
+  attr_reader :indent
 
   # io functions
   def line_no
@@ -545,7 +544,7 @@ class RubyLex
       get_read
     end
 #   throw :eof unless tk
-    p tk if $DEBUG
+    p tk if $DEBUG_RDOC
     tk
   end
   
@@ -560,7 +559,7 @@ class RubyLex
     "q" => "\'",
     "Q" => "\"",
     "x" => "\`",
-    "r" => "\/",
+    "r" => "/",
     "w" => "]"
   }
   
@@ -575,7 +574,7 @@ class RubyLex
     "\'" => TkSTRING,
     "\"" => TkSTRING,
     "\`" => TkXSTRING,
-    "\/" => TkREGEXP,
+    "/" => TkREGEXP,
     "]" => TkDSTRING
   }
   Ltype2Token.default = TkSTRING
@@ -583,11 +582,11 @@ class RubyLex
   DLtype2Token = {
     "\"" => TkDSTRING,
     "\`" => TkDXSTRING,
-    "\/" => TkDREGEXP,
+    "/" => TkDREGEXP,
   }
 
   def lex_init()
-    @OP = SLex.new
+    @OP = IRB::SLex.new
     @OP.def_rules("\0", "\004", "\032") do |chars, io|
       Token(TkEND_OF_SCRIPT).set_text(chars)
     end
@@ -1128,7 +1127,7 @@ class RubyLex
     elsif ch =~ /\W/
       lt = "\""
     else
-      RubyLex.fail SyntaxError, "unknown type of %string ('#{ch}')"
+      fail SyntaxError, "unknown type of %string ('#{ch}')"
     end
 #     if ch !~ /\W/
 #       ungetc
@@ -1334,7 +1333,7 @@ class RubyLex
 	end
       end
 
-    when "C", "c", "^"
+    when "C", "c" #, "^"
       res << ch
       if ch == "C" and (ch = getc) != "-"
 	ungetc
@@ -1426,16 +1425,23 @@ module RDoc
 
     private 
 
-    def warn(msg)
+    def make_message(msg)
       prefix = "\n" + @input_file_name + ":"
       if @scanner
         prefix << "#{@scanner.line_no}:#{@scanner.char_no}: "
       end
-      $stderr.puts prefix + msg
+      return prefix + msg
+    end
+
+    def warn(msg)
+      return if @options.quiet
+      msg = make_message msg
+      $stderr.puts msg
     end
 
     def error(msg)
-      warn msg
+      msg = make_message msg
+      $stderr.puts msg
       exit(1)
     end
 
@@ -1470,15 +1476,19 @@ module RDoc
       if tk.kind_of?(TkSYMBEG)
         set_token_position(tk.line_no, tk.char_no)
         tk1 = get_tk
-        if tk1.kind_of?(TkId) || tk1.kind_of?(TkOp)
-          tk = Token(TkSYMBOL).set_text(":" + tk1.name)
+        if tk1.kind_of?(TkId) || tk1.kind_of?(TkOp) || tk1.kind_of?(TkSTRING)
+          if tk1.respond_to?(:name)
+            tk = Token(TkSYMBOL).set_text(":" + tk1.name)
+          else
+            tk = Token(TkSYMBOL).set_text(":" + tk1.text)
+          end
           # remove the identifier we just read (we're about to
           # replace it with a symbol)
           @token_listeners.each do |obj|
             obj.pop_token
           end if @token_listeners
         else
-          warn("':' not followed by identified or operator")
+          warn("':' not followed by identifier or operator")
           tk = tk1
         end
       end
@@ -1656,21 +1666,21 @@ module RDoc
         when  TkUNTIL, TkWHILE
           nest += 1
           puts "FOUND #{tk.class} in #{container.name}, nest = #{nest}, " +
-            "line #{tk.line_no}" if $DEBUG
+            "line #{tk.line_no}" if $DEBUG_RDOC
           skip_optional_do_after_expression
 
           # 'for' is trickier
         when TkFOR
           nest += 1
           puts "FOUND #{tk.class} in #{container.name}, nest = #{nest}, " +
-            "line #{tk.line_no}" if $DEBUG
+            "line #{tk.line_no}" if $DEBUG_RDOC
           skip_for_variable
           skip_optional_do_after_expression
 
 	when TkCASE, TkDO, TkIF, TkUNLESS, TkBEGIN
 	  nest += 1
           puts "Found #{tk.class} in #{container.name}, nest = #{nest}, " +
-            "line #{tk.line_no}" if $DEBUG
+            "line #{tk.line_no}" if $DEBUG_RDOC
 
 	when TkIDENTIFIER
           if nest == 1 and current_method.nil?
@@ -1700,8 +1710,8 @@ module RDoc
 
 	when TkEND
           nest -= 1
-          puts "Found 'end' in #{container.name}, nest = #{nest}, line #{tk.line_no}" if $DEBUG
-          puts "Method = #{current_method.name}" if $DEBUG and current_method
+          puts "Found 'end' in #{container.name}, nest = #{nest}, line #{tk.line_no}" if $DEBUG_RDOC
+          puts "Method = #{current_method.name}" if $DEBUG_RDOC and current_method
 	  if nest == 0
             read_documentation_modifiers(container, CLASS_MODIFIERS)
             container.ongoing_visibility = save_visibility
@@ -1836,7 +1846,7 @@ module RDoc
 
       loop do
         puts("Param: #{tk}, #{@scanner.continue} " +
-          "#{@scanner.lex_state} #{nest}")  if $DEBUG
+          "#{@scanner.lex_state} #{nest}")  if $DEBUG_RDOC
 
         case tk
         when TkSEMICOLON
@@ -1924,9 +1934,8 @@ module RDoc
         meth.singleton = true
       else
 	unget_tk dot
-	back_tk.reverse_each do
-	  |tk|
-	  unget_tk tk
+	back_tk.reverse_each do |token|
+	  unget_tk token
 	end
 	name = name_t.name
 
@@ -2033,7 +2042,7 @@ module RDoc
 
       loop do
         puts("Param: #{tk.inspect}, #{@scanner.continue} " +
-          "#{@scanner.lex_state} #{nest}")  if $DEBUG
+          "#{@scanner.lex_state} #{nest}")  if $DEBUG_RDOC
         case tk
         when TkSEMICOLON
           break
@@ -2089,7 +2098,7 @@ module RDoc
 
       loop do
         puts("\nWhile: #{tk}, #{@scanner.continue} " +
-          "#{@scanner.lex_state} #{nest}") if $DEBUG
+          "#{@scanner.lex_state} #{nest}") if $DEBUG_RDOC
         case tk
         when TkSEMICOLON
           break
@@ -2160,7 +2169,7 @@ module RDoc
 
       loop do
         puts("Call param: #{tk}, #{@scanner.continue} " +
-          "#{@scanner.lex_state} #{nest}") if $DEBUG
+          "#{@scanner.lex_state} #{nest}") if $DEBUG_RDOC
         case tk
         when TkSEMICOLON
           break
@@ -2242,7 +2251,7 @@ module RDoc
     
     def read_directive(allowed)
       tk = get_tk
-      puts "directive: #{tk.inspect}" if $DEBUG
+      puts "directive: #{tk.inspect}" if $DEBUG_RDOC
       result = nil
       if tk.kind_of?(TkCOMMENT) 
         if tk.text =~ /\s*:?(\w+):\s*(.*)/
@@ -2327,7 +2336,7 @@ module RDoc
 
         when "section"
           context.set_current_section(param, comment)
-          comment.replace("") # 1.8 doesn't support #clear
+          comment.clear
           break 
         else
           warn "Unrecognized directive '#{directive}'"
@@ -2551,7 +2560,7 @@ module RDoc
 	    break
 	  when TkCOMMA
 	  else
-	    warn("unexpected token: '#{tk2.inspect}'") if $DEBBUG
+           warn("unexpected token: '#{tk2.inspect}'") if $DEBUG_RDOC
 	    break
 	  end
 	end
@@ -2589,7 +2598,7 @@ module RDoc
       when TkSTRING
 	eval @read[-1]
       else
-	warn("Expected symbol or string, got #{tk.inspect}") if $DEBUG
+	warn("Expected symbol or string, got #{tk.inspect}") if $DEBUG_RDOC
 	nil
       end
     end
