@@ -2,8 +2,8 @@
 
   acosh.c -
 
-  $Author: eban $
-  $Date: 2003/10/18 14:04:18 $
+  $Author: shyouhei $
+  $Date: 2007-02-13 08:01:19 +0900 (Tue, 13 Feb 2007) $
   created at: Fri Apr 12 00:34:17 JST 2002
 
   public domain rewrite of acosh(3), asinh(3) and atanh(3)
@@ -33,8 +33,7 @@
 
 #ifndef HAVE_ACOSH
 double
-acosh(x)
-    double x;
+acosh(double x)
 {
     if (x < 1)
 	x = -1;			/* NaN */
@@ -50,8 +49,7 @@ acosh(x)
 
 #ifndef HAVE_ASINH
 double
-asinh(x)
-    double x;
+asinh(double x)
 {
     int neg = x < 0;
     double z = fabs(x);
@@ -74,8 +72,7 @@ asinh(x)
 
 #ifndef HAVE_ATANH
 double
-atanh(x)
-    double x;
+atanh(double x)
 {
     int neg = x < 0;
     double z = fabs(x);
