@@ -2,31 +2,30 @@
 
   sprintf.c -
 
-  $Author: knu $
-  $Date: 2008-05-19 14:44:47 +0900 (Mon, 19 May 2008) $
+  $Author: akr $
+  $Date: 2007-12-21 11:31:11 +0900 (Fri, 21 Dec 2007) $
   created at: Fri Oct 15 10:39:26 JST 1993
 
-  Copyright (C) 1993-2003 Yukihiro Matsumoto
+  Copyright (C) 1993-2007 Yukihiro Matsumoto
   Copyright (C) 2000  Network Applied Communication Laboratory, Inc.
   Copyright (C) 2000  Information-technology Promotion Agency, Japan
 
 **********************************************************************/
 
-#include "ruby.h"
-#include "re.h"
-#include <ctype.h>
+#include "ruby/ruby.h"
+#include "ruby/re.h"
+#include "ruby/encoding.h"
 #include <math.h>
+#include <stdarg.h>
 
 #define BIT_DIGITS(N)   (((N)*146)/485 + 1)  /* log2(10) =~ 146/485 */
 #define BITSPERDIG (SIZEOF_BDIGITS*CHAR_BIT)
 #define EXTENDSIGN(n, l) (((~0 << (n)) >> (((n)*(l)) % BITSPERDIG)) & ~(~0 << (n)))
 
-static void fmt_setup _((char*,int,int,int,int));
+static void fmt_setup(char*,int,int,int,int);
 
 static char*
-remove_sign_bits(str, base)
-    char *str;
-    int base;
+remove_sign_bits(char *str, int base)
 {
     char *s, *t;
     
@@ -57,9 +56,7 @@ remove_sign_bits(str, base)
 }
 
 static char
-sign_bits(base, p)
-    int base;
-    const char *p;
+sign_bits(int base, const char *p)
 {
     char c = '.';
 
@@ -91,7 +88,7 @@ sign_bits(base, p)
 	bsiz*=2;\
     }\
     rb_str_resize(result, bsiz);\
-    buf = RSTRING(result)->ptr;\
+    buf = RSTRING_PTR(result);\
 } while (0)
 
 #define PUSH(s, l) do { \
@@ -100,6 +97,12 @@ sign_bits(base, p)
     blen += (l);\
 } while (0)
 
+#define FILL(c, l) do { \
+    CHECK(l);\
+    memset(&buf[blen], c, l);\
+    blen += (l);\
+} while (0)
+
 #define GETARG() (nextvalue != Qundef ? nextvalue : \
     posarg < 0 ? \
     (rb_raise(rb_eArgError, "unnumbered(%d) mixed with numbered", nextarg), 0) : \
@@ -114,7 +117,7 @@ sign_bits(base, p)
     ((nth >= argc) ? (rb_raise(rb_eArgError, "too few arguments"), 0) : argv[nth])
 
 #define GETNUM(n, val) \
-    for (; p < end && ISDIGIT(*p); p++) { \
+    for (; p < end && rb_enc_isdigit(*p, enc); p++) {	\
 	int next_n = 10 * n + (*p - '0'); \
         if (next_n / 10 != n) {\
 	    rb_raise(rb_eArgError, #val " too big"); \
@@ -245,11 +248,15 @@ sign_bits(base, p)
  */
 
 VALUE
-rb_f_sprintf(argc, argv)
-    int argc;
-    VALUE *argv;
+rb_f_sprintf(int argc, const VALUE *argv)
 {
-    VALUE fmt;
+    return rb_str_format(argc - 1, argv + 1, GETNTHARG(0));
+}
+
+VALUE
+rb_str_format(int argc, const VALUE *argv, VALUE fmt)
+{
+    rb_encoding *enc;
     const char *p, *end;
     char *buf;
     int blen, bsiz;
@@ -278,16 +285,20 @@ rb_f_sprintf(argc, argv)
 	rb_raise(rb_eArgError, "flag after precision"); \
     }
 
-    fmt = GETNTHARG(0);
+    ++argc;
+    --argv;
     if (OBJ_TAINTED(fmt)) tainted = 1;
     StringValue(fmt);
+    enc = rb_enc_get(fmt);
     fmt = rb_str_new4(fmt);
-    p = RSTRING(fmt)->ptr;
-    end = p + RSTRING(fmt)->len;
+    p = RSTRING_PTR(fmt);
+    end = p + RSTRING_LEN(fmt);
     blen = 0;
     bsiz = 120;
     result = rb_str_buf_new(bsiz);
-    buf = RSTRING(result)->ptr;
+    rb_enc_copy(result, fmt);
+    buf = RSTRING_PTR(result);
+    memset(buf, 0, bsiz);
 
     for (; p < end; p++) {
 	const char *t;
@@ -306,7 +317,7 @@ rb_f_sprintf(argc, argv)
       retry:
 	switch (*p) {
 	  default:
-	    if (ISPRINT(*p))
+	    if (rb_enc_isprint(*p, enc))
 		rb_raise(rb_eArgError, "malformed format string - %%%c", *p);
 	    else
 		rb_raise(rb_eArgError, "malformed format string");
@@ -395,7 +406,7 @@ rb_f_sprintf(argc, argv)
 	    p--;
 	  case '%':
 	    if (flags != FNONE) {
-		rb_raise(rb_eArgError, "illegal format character - %%");
+		rb_raise(rb_eArgError, "invalid format character - %%");
 	    }
 	    PUSH("%", 1);
 	    break;
@@ -403,15 +414,40 @@ rb_f_sprintf(argc, argv)
 	  case 'c':
 	    {
 		VALUE val = GETARG();
-		char c;
+		VALUE tmp;
+		int c, n;
 
-		if (!(flags & FMINUS))
-		    while (--width > 0)
-			PUSH(" ", 1);
-		c = NUM2INT(val) & 0xff;
-		PUSH(&c, 1);
-		while (--width > 0)
-		    PUSH(" ", 1);
+		tmp = rb_check_string_type(val);
+		if (!NIL_P(tmp)) {
+		    if (rb_enc_strlen(RSTRING_PTR(tmp),RSTRING_END(tmp),enc) != 1) {
+			rb_raise(rb_eArgError, "%%c requires a character");
+		    }
+		    c = rb_enc_codepoint(RSTRING_PTR(tmp), RSTRING_END(tmp), enc);
+		}
+		else {
+		    c = NUM2INT(val);
+		}
+		n = rb_enc_codelen(c, enc);
+		if (n == 0) {
+		    rb_raise(rb_eArgError, "invalid character");
+		}
+		if (!(flags & FWIDTH)) {
+		    CHECK(n);
+		    rb_enc_mbcput(c, &buf[blen], enc);
+		    blen += n;
+		}
+		else if ((flags & FMINUS)) {
+		    CHECK(n);
+		    rb_enc_mbcput(c, &buf[blen], enc);
+		    blen += n;
+		    FILL(' ', width-1);
+		}
+		else {
+		    FILL(' ', width-1);
+		    CHECK(n);
+		    rb_enc_mbcput(c, &buf[blen], enc);
+		    blen += n;
+		}
 	    }
 	    break;
 
@@ -419,30 +455,42 @@ rb_f_sprintf(argc, argv)
 	  case 'p':
 	    {
 		VALUE arg = GETARG();
-		long len;
+		long len, slen;
 
 		if (*p == 'p') arg = rb_inspect(arg);
 		str = rb_obj_as_string(arg);
 		if (OBJ_TAINTED(str)) tainted = 1;
-		len = RSTRING(str)->len;
+		len = RSTRING_LEN(str);
+		enc = rb_enc_check(result, str);
+		if (flags&(FPREC|FWIDTH)) {
+		    slen = rb_enc_strlen(RSTRING_PTR(str),RSTRING_END(str),enc);
+		    if (slen < 0) {
+			rb_raise(rb_eArgError, "invalid mbstring sequence");
+		    }
+		}
 		if (flags&FPREC) {
-		    if (prec < len) {
-			len = prec;
+		    if (prec < slen) {
+			char *p = rb_enc_nth(RSTRING_PTR(str), RSTRING_END(str),
+					     prec, enc);
+			slen = prec;
+			len = p - RSTRING_PTR(str);
 		    }
 		}
 		/* need to adjust multi-byte string pos */
 		if (flags&FWIDTH) {
-		    if (width > len) {
-			CHECK(width);
-			width -= len;
+		    if (width > slen) {
+			width -= slen;
 			if (!(flags&FMINUS)) {
+			    CHECK(width);
 			    while (width--) {
 				buf[blen++] = ' ';
 			    }
 			}
+			CHECK(len);
 			memcpy(&buf[blen], RSTRING_PTR(str), len);
 			blen += len;
 			if (flags&FMINUS) {
+			    CHECK(width);
 			    while (width--) {
 				buf[blen++] = ' ';
 			    }
@@ -450,7 +498,8 @@ rb_f_sprintf(argc, argv)
 			break;
 		    }
 		}
-		PUSH(RSTRING(str)->ptr, len);
+		PUSH(RSTRING_PTR(str), len);
+		rb_enc_associate(result, enc);
 	    }
 	    break;
 
@@ -471,20 +520,17 @@ rb_f_sprintf(argc, argv)
 		long v = 0;
 		int base, bignum = 0;
 		int len, pos;
-		volatile VALUE tmp;
-                volatile VALUE tmp1;
 
 		switch (*p) {
 		  case 'd':
 		  case 'i':
+		  case 'u':
 		    sign = 1; break;
 		  case 'o':
 		  case 'x':
 		  case 'X':
 		  case 'b':
 		  case 'B':
-		  case 'u':
-		  default:
 		    if (flags&(FPLUS|FSPACE)) sign = 1;
 		    break;
 		}
@@ -509,7 +555,11 @@ rb_f_sprintf(argc, argv)
 	      bin_retry:
 		switch (TYPE(val)) {
 		  case T_FLOAT:
-		    val = rb_dbl2big(RFLOAT(val)->value);
+		    if (FIXABLE((long)RFLOAT_VALUE(val))) {
+			val = LONG2FIX((long)RFLOAT_VALUE(val));
+			goto bin_retry;
+		    }
+		    val = rb_dbl2big(RFLOAT_VALUE(val));
 		    if (FIXNUM_P(val)) goto bin_retry;
 		    bignum = 1;
 		    break;
@@ -566,96 +616,92 @@ rb_f_sprintf(argc, argv)
 			}
 			sprintf(fbuf, "%%l%c", c);
 			sprintf(nbuf, fbuf, v);
-			s = nbuf;
-			goto format_integer;
 		    }
-		    s = nbuf;
-		    if (v < 0) {
-			if (base == 10) {
-			    rb_warning("negative number for %%u specifier");
-			}
-			if (!(flags&(FPREC|FZERO))) {
+		    else {
+			s = nbuf;
+			if (v < 0) {
 			    strcpy(s, "..");
 			    s += 2;
 			}
-		    }
-		    sprintf(fbuf, "%%l%c", *p == 'X' ? 'x' : *p);
-		    sprintf(s, fbuf, v);
-		    if (v < 0) {
-			char d = 0;
+			sprintf(fbuf, "%%l%c", *p == 'X' ? 'x' : *p);
+			sprintf(s, fbuf, v);
+			if (v < 0) {
+			    char d = 0;
 
-			remove_sign_bits(s, base);
-			switch (base) {
-			  case 16:
-			    d = 'f'; break;
-			  case 8:
-			    d = '7'; break;
-			}
-			if (d && *s != d) {
-			    memmove(s+1, s, strlen(s)+1);
-			    *s = d;
+			    remove_sign_bits(s, base);
+			    switch (base) {
+			      case 16:
+				d = 'f'; break;
+			      case 8:
+				d = '7'; break;
+			    }
+			    if (d && *s != d) {
+				memmove(s+1, s, strlen(s)+1);
+				*s = d;
+			    }
 			}
 		    }
 		    s = nbuf;
-		    goto format_integer;
+		}
+		else {
+		    if (sign) {
+			tmp = rb_big2str(val, base);
+			s = RSTRING_PTR(tmp);
+			if (s[0] == '-') {
+			    s++;
+			    sc = '-';
+			    width--;
+			}
+			else if (flags & FPLUS) {
+			    sc = '+';
+			    width--;
+			}
+			else if (flags & FSPACE) {
+			    sc = ' ';
+			    width--;
+			}
+		    }
+		    else {
+                        volatile VALUE tmp1;
+			if (!RBIGNUM_SIGN(val)) {
+			    val = rb_big_clone(val);
+			    rb_big_2comp(val);
+			}
+			tmp1 = tmp = rb_big2str0(val, base, RBIGNUM_SIGN(val));
+			s = RSTRING_PTR(tmp);
+			if (*s == '-') {
+			    if (base == 10) {
+				rb_warning("negative number for %%u specifier");
+			    }
+			    remove_sign_bits(++s, base);
+			    tmp = rb_str_new(0, 3+strlen(s));
+			    t = RSTRING_PTR(tmp);
+			    if (!(flags&(FPREC|FZERO))) {
+				strcpy(t, "..");
+				t += 2;
+			    }
+			    switch (base) {
+			      case 16:
+				if (s[0] != 'f') strcpy(t++, "f"); break;
+			      case 8:
+				if (s[0] != '7') strcpy(t++, "7"); break;
+			      case 2:
+				if (s[0] != '1') strcpy(t++, "1"); break;
+			    }
+			    strcpy(t, s);
+			    s  = RSTRING_PTR(tmp);
+			}
+		    }
 		}
 
-		if (sign) {
-		    tmp = rb_big2str(val, base);
-		    s = RSTRING(tmp)->ptr;
-		    if (s[0] == '-') {
-			s++;
-			sc = '-';
-                        width--;
-		    }
-		    else if (flags & FPLUS) {
-			sc = '+';
-                        width--;
-		    }
-		    else if (flags & FSPACE) {
-			sc = ' ';
-                        width--;
-		    }
-		    goto format_integer;
-		}
-		if (!RBIGNUM(val)->sign) {
-		    val = rb_big_clone(val);
-		    rb_big_2comp(val);
-		}
-		tmp1 = tmp = rb_big2str0(val, base, RBIGNUM(val)->sign);
-		s = RSTRING(tmp)->ptr;
-		if (*s == '-') {
-		    if (base == 10) {
-			rb_warning("negative number for %%u specifier");
-		    }
-		    remove_sign_bits(++s, base);
-		    tmp = rb_str_new(0, 3+strlen(s));
-		    t = RSTRING(tmp)->ptr;
-		    if (!(flags&(FPREC|FZERO))) {
-			strcpy(t, "..");
-			t += 2;
-		    }
-		    switch (base) {
-		      case 16:
-			if (s[0] != 'f') strcpy(t++, "f"); break;
-		      case 8:
-			if (s[0] != '7') strcpy(t++, "7"); break;
-		      case 2:
-			if (s[0] != '1') strcpy(t++, "1"); break;
-		    }
-		    strcpy(t, s);
-		    bignum = 2;
-		}
-		s = RSTRING(tmp)->ptr;
-
-	      format_integer:
 		pos = -1;
 		len = strlen(s);
 
 		if (*p == 'X') {
 		    char *pp = s;
-		    while (*pp) {
-			*pp = toupper(*pp);
+		    int c;
+		    while ((c = (int)(unsigned char)*pp) != 0) {
+			*pp = rb_enc_toupper(c, enc);
 			pp++;
 		    }
 		}
@@ -688,7 +734,7 @@ rb_f_sprintf(argc, argv)
 		else {
 		    char c;
 
-		    if (!sign && bignum && !RBIGNUM(val)->sign)
+		    if (!sign && bignum && !RBIGNUM_SIGN(val))
 			c = sign_bits(base, p);
 		    else
 			c = '0';
@@ -715,10 +761,9 @@ rb_f_sprintf(argc, argv)
 		int i, need = 6;
 		char fbuf[32];
 
-		fval = RFLOAT(rb_Float(val))->value;
-#if defined(_WIN32) && !defined(__BORLANDC__)
+		fval = RFLOAT_VALUE(rb_Float(val));
 		if (isnan(fval) || isinf(fval)) {
-		    char *expr;
+		    const char *expr;
 
 		    if  (isnan(fval)) {
 			expr = "NaN";
@@ -729,8 +774,6 @@ rb_f_sprintf(argc, argv)
 		    need = strlen(expr);
 		    if ((!isnan(fval) && fval < 0.0) || (flags & FPLUS))
 			need++;
-		    else if (flags & FSPACE)
-			need++;
 		    if ((flags & FWIDTH) && need < width)
 			need = width;
 
@@ -774,7 +817,7 @@ rb_f_sprintf(argc, argv)
 		    blen += strlen(&buf[blen]);
 		    break;
 		}
-#endif	/* defined(_WIN32) && !defined(__BORLANDC__) */
+
 		fmt_setup(fbuf, *p, flags, width, prec);
 		need = 0;
 		if (*p != 'e' && *p != 'E') {
@@ -812,10 +855,7 @@ rb_f_sprintf(argc, argv)
 }
 
 static void
-fmt_setup(buf, c, flags, width, prec)
-    char *buf;
-    int c;
-    int flags, width, prec;
+fmt_setup(char *buf, int c, int flags, int width, int prec)
 {
     *buf++ = '%';
     if (flags & FSHARP) *buf++ = '#';
@@ -837,3 +877,83 @@ fmt_setup(buf, c, flags, width, prec)
     *buf++ = c;
     *buf = '\0';
 }
+
+#undef FILE
+#define FILE rb_printf_buffer
+#define __sbuf rb_printf_sbuf
+#define __sFILE rb_printf_sfile
+#undef feof
+#undef ferror
+#undef clearerr
+#undef fileno
+#if SIZEOF_LONG < SIZEOF_VOIDP
+# if  SIZEOF_LONG_LONG == SIZEOF_VOIDP
+#  define _HAVE_SANE_QUAD_
+#  define _HAVE_LLP64_
+#  define quad_t LONG_LONG
+#  define u_quad_t unsigned LONG_LONG
+# endif
+#endif
+#undef vsnprintf
+#undef snprintf
+#include "missing/vsnprintf.c"
+
+static int
+ruby__sfvwrite(register rb_printf_buffer *fp, register struct __suio *uio)
+{
+    struct __siov *iov;
+    VALUE result = (VALUE)fp->_bf._base;
+    char *buf = (char*)fp->_p;
+    size_t len, n;
+    int blen = buf - RSTRING_PTR(result), bsiz = fp->_w;
+
+    if (RBASIC(result)->klass) {
+	rb_raise(rb_eRuntimeError, "rb_vsprintf reentered");
+    }
+    if ((len = uio->uio_resid) == 0)
+	return 0;
+    CHECK(len);
+    buf += blen;
+    fp->_w = bsiz;
+    for (iov = uio->uio_iov; len > 0; ++iov) {
+	MEMCPY(buf, iov->iov_base, char, n = iov->iov_len);
+	buf += n;
+	len -= n;
+    }
+    fp->_p = (unsigned char *)buf;
+    return 0;
+}
+
+VALUE
+rb_vsprintf(const char *fmt, va_list ap)
+{
+    rb_printf_buffer f;
+    VALUE result;
+
+    f._flags = __SWR | __SSTR;
+    f._bf._size = 0;
+    f._w = 120;
+    result = rb_str_buf_new(f._w);
+    f._bf._base = (unsigned char *)result;
+    f._p = (unsigned char *)RSTRING_PTR(result);
+    RBASIC(result)->klass = 0;
+    f.vwrite = ruby__sfvwrite;
+    BSD_vfprintf(&f, fmt, ap);
+    RBASIC(result)->klass = rb_cString;
+    rb_str_resize(result, (char *)f._p - RSTRING_PTR(result));
+
+    return result;
+}
+
+VALUE
+rb_sprintf(const char *format, ...)
+{
+    VALUE result;
+    va_list ap;
+
+    va_start(ap, format);
+    result = rb_vsprintf(format, ap);
+    va_end(ap);
+
+    return result;
+}
