@@ -1,6 +1,6 @@
 # -*- mode: ruby; ruby-indent-level: 4; tab-width: 4 -*-
 #												vim:sw=4:ts=4
-# $Id: test_yaml.rb 11708 2007-02-12 23:01:19Z shyouhei $
+# $Id: test_yaml.rb 14073 2007-12-01 17:24:13Z akr $
 #
 require 'test/unit'
 require 'yaml'
@@ -54,7 +54,7 @@ class YAML_Unit_Tests < Test::Unit::TestCase
 			hour = zone[0,3].to_i * 3600
 			min = zone[3,2].to_i * 60
 			ofs = (hour + min)
-			val = Time.at( val.to_f - ofs )
+			val = Time.at( val.tv_sec - ofs, val.tv_nsec / 1000.0 )
 		end
 		return val
 	end
@@ -1082,27 +1082,27 @@ EOY
 				book_struct.new( "This should be the ISBN", "but I have another struct here", 2002, "None" ) 
 			  ) ], <<EOY
 - !ruby/struct:BookStruct
-  author: Yukihiro Matsumoto
-  title: Ruby in a Nutshell
-  year: 2002
-  isbn: 0-596-00214-9
+  :author: Yukihiro Matsumoto
+  :title: Ruby in a Nutshell
+  :year: 2002
+  :isbn: 0-596-00214-9
 - !ruby/struct:BookStruct
-  author:
+  :author:
     - Dave Thomas
     - Andy Hunt
-  title: The Pickaxe
-  year: 2002
-  isbn: !ruby/struct:BookStruct
-    author: This should be the ISBN
-    title: but I have another struct here
-    year: 2002
-    isbn: None
+  :title: The Pickaxe
+  :year: 2002
+  :isbn: !ruby/struct:BookStruct
+    :author: This should be the ISBN
+    :title: but I have another struct here
+    :year: 2002
+    :isbn: None
 EOY
 		)
 
         assert_to_yaml( YAML_Tests::StructTest.new( 123 ), <<EOY )
 --- !ruby/struct:YAML_Tests::StructTest
-c: 123
+:c: 123
 EOY
 
 	end
@@ -1190,6 +1190,7 @@ EOY
         #
         require 'yaml'
         t = Time.now
+        t = Time.at(t.tv_sec, t.tv_usec)
         5.times do
             assert_cycle(t)
         end
@@ -1272,6 +1273,19 @@ EOY
       assert_equal([{}], o.keys)
     end
 
+    #
+    # contributed by riley lynch [ruby-Bugs-8548]
+    #
+    def test_object_id_collision
+      omap = YAML::Omap.new
+      1000.times { |i| omap["key_#{i}"] = { "value" => i } }
+      raise "id collision in ordered map" if omap.to_yaml =~ /id\d+/
+    end
+
+    def test_normal_exit
+      YAML.load("2000-01-01 00:00:00.#{"0"*1000} +00:00\n")
+      # '[ruby-core:13735]'
+    end
 end
 
 if $0 == __FILE__
