@@ -2,15 +2,15 @@
 
   util.c -
 
-  $Author: knu $
-  $Date: 2008-05-31 20:44:49 +0900 (Sat, 31 May 2008) $
+  $Author: nobu $
+  $Date: 2007-08-27 15:33:12 +0900 (Mon, 27 Aug 2007) $
   created at: Fri Mar 10 17:22:34 JST 1995
 
-  Copyright (C) 1993-2003 Yukihiro Matsumoto
+  Copyright (C) 1993-2007 Yukihiro Matsumoto
 
 **********************************************************************/
 
-#include "ruby.h"
+#include "ruby/ruby.h"
 
 #include <ctype.h>
 #include <stdio.h>
@@ -21,17 +21,19 @@
 #ifdef _WIN32
 #include "missing/file.h"
 #endif
-
-#include "util.h"
-#ifndef HAVE_STRING_H
-char *strchr _((char*,char));
+#if defined(__CYGWIN32__)
+#define _open open
+#define _close close
+#define _unlink unlink
+#define _access access
+#elif defined(_WIN32)
+#include <io.h>
 #endif
 
+#include "ruby/util.h"
+
 unsigned long
-scan_oct(start, len, retlen)
-    const char *start;
-    int len;
-    int *retlen;
+ruby_scan_oct(const char *start, int len, int *retlen)
 {
     register const char *s = start;
     register unsigned long retval = 0;
@@ -45,10 +47,7 @@ scan_oct(start, len, retlen)
 }
 
 unsigned long
-scan_hex(start, len, retlen)
-    const char *start;
-    int len;
-    int *retlen;
+ruby_scan_hex(const char *start, int len, int *retlen)
 {
     static char hexdigit[] = "0123456789abcdef0123456789ABCDEF";
     register const char *s = start;
@@ -147,19 +146,17 @@ scan_hex(start, len, retlen)
  */
 
 
-static int valid_filename(char *s);
+static int valid_filename(const char *s);
 
-static char suffix1[] = ".$$$";
-static char suffix2[] = ".~~~";
+static const char suffix1[] = ".$$$";
+static const char suffix2[] = ".~~~";
 
 #define ext (&buf[1000])
 
 #define strEQ(s1,s2) (strcmp(s1,s2) == 0)
 
 void
-ruby_add_suffix(str, suffix)
-    VALUE str;
-    char *suffix;
+ruby_add_suffix(VALUE str, const char *suffix)
 {
     int baselen;
     int extlen = strlen(suffix);
@@ -167,26 +164,26 @@ ruby_add_suffix(str, suffix)
     long slen;
     char buf[1024];
 
-    if (RSTRING(str)->len > 1000)
+    if (RSTRING_LEN(str) > 1000)
         rb_fatal("Cannot do inplace edit on long filename (%ld characters)",
-		 RSTRING(str)->len);
+		 RSTRING_LEN(str));
 
 #if defined(DJGPP) || defined(__CYGWIN32__) || defined(_WIN32)
     /* Style 0 */
-    slen = RSTRING(str)->len;
+    slen = RSTRING_LEN(str);
     rb_str_cat(str, suffix, extlen);
 #if defined(DJGPP)
     if (_USE_LFN) return;
 #else
-    if (valid_filename(RSTRING(str)->ptr)) return;
+    if (valid_filename(RSTRING_PTR(str))) return;
 #endif
 
     /* Fooey, style 0 failed.  Fix str before continuing. */
-    RSTRING(str)->ptr[RSTRING(str)->len = slen] = '\0';
+    rb_str_resize(str, slen);
 #endif
 
     slen = extlen;
-    t = buf; baselen = 0; s = RSTRING(str)->ptr;
+    t = buf; baselen = 0; s = RSTRING_PTR(str);
     while ((*t = *s) && *s != '.') {
 	baselen++;
 	if (*s == '\\' || *s == '/') baselen = 0;
@@ -195,7 +192,7 @@ ruby_add_suffix(str, suffix)
     p = t;
 
     t = ext; extlen = 0;
-    while (*t++ = *s++) extlen++;
+    while ((*t++ = *s++) != 0) extlen++;
     if (extlen == 0) { ext[0] = '.'; ext[1] = 0; extlen++; }
 
     if (*suffix == '.') {        /* Style 1 */
@@ -224,28 +221,32 @@ fallback:
 	(void)memcpy(p, strEQ(ext, suffix1) ? suffix2 : suffix1, 5);
     }
     rb_str_resize(str, strlen(buf));
-    memcpy(RSTRING(str)->ptr, buf, RSTRING(str)->len);
+    memcpy(RSTRING_PTR(str), buf, RSTRING_LEN(str));
 }
 
 #if defined(__CYGWIN32__) || defined(_WIN32)
 static int 
-valid_filename(char *s)
+valid_filename(const char *s)
 {
     int fd;
 
+    /*
+    // if the file exists, then it's a valid filename!
+    */
+
+    if (_access(s, 0) == 0) {
+	return 1;
+    }
+
     /*
     // It doesn't exist, so see if we can open it.
     */
 
-    if ((fd = _open(s, O_CREAT|O_EXCL, 0666)) >= 0) {
+    if ((fd = _open(s, O_CREAT, 0666)) >= 0) {
 	_close(fd);
 	_unlink(s);	/* don't leave it laying around */
 	return 1;
     }
-    else if (errno == EEXIST) {
-	/* if the file exists, then it's a valid filename! */
-	return 1;
-    }
     return 0;
 }
 #endif
@@ -349,7 +350,7 @@ __crt0_glob_function(char *path)
     strncpy(buf, path, len);
     buf[len] = '\0';
 
-    for (p = buf; *p; p += mblen(p, MB_CUR_MAX))
+    for (p = buf; *p; p += mblen(p, RUBY_MBCHAR_MAXSIZE))
 	if (*p == '\\')
 	    *p = '/';
 
@@ -400,9 +401,7 @@ __crt0_glob_function(char *path)
 
 #define mmarg mmkind, size, high, low
 
-static void mmswap_(a, b, mmarg)
-    register char *a, *b;
-    int mmarg;
+static void mmswap_(register char *a, register char *b, int mmkind, int size, int high, int low)
 {
  register int s;
  if (a == b) return;
@@ -427,9 +426,7 @@ static void mmswap_(a, b, mmarg)
 }
 #define mmswap(a,b) mmswap_((a),(b),mmarg)
 
-static void mmrot3_(a, b, c, mmarg)
-    register char *a, *b, *c;
-    int mmarg;
+static void mmrot3_(register char *a, register char *b, register char *c, int mmkind, int size, int high, int low)
 {
  register int s;
  if (mmkind >= 0) {
@@ -470,12 +467,9 @@ typedef struct { char *LL, *RR; } stack_node; /* Stack structure for L,l,R,r */
                        ((*cmp)(b,c,d)<0 ? b : ((*cmp)(a,c,d)<0 ? c : a)) : \
                        ((*cmp)(b,c,d)>0 ? b : ((*cmp)(a,c,d)<0 ? a : c)))
 
-void ruby_qsort (base, nel, size, cmp, d)
-     void* base;
-     const int nel;
-     const int size;
-     int (*cmp)();
-     void *d;
+void
+ruby_qsort(void* base, const int nel, const int size,
+	   int (*cmp)(const void*, const void*, void*), void *d)
 {
   register char *l, *r, *m;          	/* l,r:left,right group   m:median point */
   register int  t, eq_l, eq_r;       	/* eq_l: all items in left group are equal to S */
@@ -623,8 +617,7 @@ void ruby_qsort (base, nel, size, cmp, d)
 }
 
 char *
-ruby_strdup(str)
-    const char *str;
+ruby_strdup(const char *str)
 {
     char *tmp;
     int len = strlen(str) + 1;
@@ -636,7 +629,7 @@ ruby_strdup(str)
 }
 
 char *
-ruby_getcwd()
+ruby_getcwd(void)
 {
 #ifdef HAVE_GETCWD
     int size = 200;
@@ -664,7 +657,6 @@ ruby_getcwd()
     return buf;
 }
 
-
 /****************************************************************
  *
  * The author of this software is David M. Gay.
@@ -3100,7 +3092,7 @@ rv_alloc(int i)
 }
 
 static char *
-nrv_alloc(const char *s, char **rve, int n)
+nrv_alloc(char *s, char **rve, int n)
 {
     char *rv, *t;
 
@@ -3833,24 +3825,6 @@ ret1:
         *rve = s;
     return s0;
 }
-
-void
-ruby_each_words(const char *str, void (*func)(const char*, int, void*), void *arg)
-{
-    const char *end;
-    int len;
-
-    if (!str) return;
-    for (; *str; str = end) {
-	while (ISSPACE(*str) || *str == ',') str++;
-	if (!*str) break;
-	end = str;
-	while (*end && !ISSPACE(*end) && *end != ',') end++;
-	len = end - str;
-	(*func)(str, len, arg);
-    }
-}
-
 #ifdef __cplusplus
 }
 #endif
